@@ -1,22 +1,24 @@
 from flask import session
 from flask.ext.babelpkg import Babel
+from flask.ext.appbuilder import translations
 from .views import LocaleView
+from ..basemanager import BaseManager
 
-
-class BabelManager(object):
+class BabelManager(BaseManager):
 
     babel = None
     babel_default_locale = ''
     locale_view = None
 
-    def __init__(self, app, pkg_translations):
-        self.babel = Babel(app, pkg_translations)
-        self.babel_default_locale = self._get_default_locale(app)
+    def __init__(self, baseapp):
+        super(BabelManager, self).__init__(baseapp)
+        self.babel = Babel(baseapp.get_app, translations)
+        self.babel_default_locale = self._get_default_locale(baseapp.get_app)
         self.babel.locale_selector_func = self.get_locale
 
-    def register_views(self, baseapp):
+    def register_views(self):
         self.locale_view = LocaleView()
-        baseapp.add_view_no_menu(self.locale_view)
+        self.baseapp.add_view_no_menu(self.locale_view)
 
     @staticmethod
     def _get_default_locale(app):
