@@ -1,9 +1,7 @@
 import logging
 
 from flask import Blueprint, url_for
-from flask.ext.babelpkg import gettext as _gettext, lazy_gettext
 from flask.ext.appbuilder.babel.manager import BabelManager
-from flask.ext.appbuilder import translations
 from flask.ext.appbuilder import Base
 from .views import IndexView
 from .filters import TemplateFilters
@@ -48,7 +46,6 @@ class BaseApp(object):
 
     languages = None
     admin = None
-    _gettext = _gettext
 
     def __init__(self, app, db,
                  menu=None,
@@ -75,8 +72,8 @@ class BaseApp(object):
         self.app = app
         self.db = db
 
-        self.sm = SecurityManager(app, db.session)
-        self.bm = BabelManager(app, pkg_translations=translations)
+        self.sm = SecurityManager(self)
+        self.bm = BabelManager(self)
 
         if menu:
             self.menu = menu
@@ -99,6 +96,14 @@ class BaseApp(object):
         engine = self.db.session.get_bind(mapper=None, clause=None)
         Base.metadata.create_all(engine)
 
+    @property
+    def get_app(self):
+        return self.app
+
+    @property
+    def get_session(self):
+        return self.db.session
+
     def _init_config_parameters(self):
         if 'APP_NAME' in self.app.config:
             self.app_name = self.app.config['APP_NAME']
@@ -129,13 +134,8 @@ class BaseApp(object):
     def _add_admin_views(self):
         self.indexview = self.indexview()
         self.add_view_no_menu(self.indexview)
-        self.bm.register_views(self)
-        self.sm.register_views(self)
-
-    def _init_view_session(self, baseview_class):
-        if baseview_class.datamodel.session is None:
-            baseview_class.datamodel.session = self.db.session
-        return baseview_class()
+        self.bm.register_views()
+        self.sm.register_views()
 
     def _add_permissions_menu(self, name):
         try:
@@ -150,22 +150,34 @@ class BaseApp(object):
             for item in category.childs:
                 self._add_permissions_menu(item.name)
 
+    def _check_and_init(self, baseview):
+        # If class if not instantiated, instantiate it and add security db session.
+        if hasattr(baseview, '__call__'):
+            if hasattr(baseview, 'datamodel'):
+                if baseview.datamodel.session is None:
+                    baseview.datamodel.session = self.db.session
+            baseview = baseview()
+        return baseview
+
     def add_view(self, baseview, name, href="", icon="", label="", category="", category_icon="", category_label=""):
         """
             Add your views associated with menus using this method.
             
             :param baseview:
-                A BaseView type class instantiated.
+                A BaseView type class instantiated or not.
+                This method will instantiate the class for you if needed.
             :param name:
                 The string name that identifies the menu.
             :param href:
-                Override the generated href for the menu. if non provided default_view from view will be set as href.
+                Override the generated href for the menu.
+                if non provided default_view from view will be set as href.
             :param icon:
                 Font-Awesome icon name, optional.
             :param label:
                 The label that will be displayed on the menu, if absent param name will be used
             :param category:
-                The menu category where the menu will be included, if non provided the view will be acessible as a top menu.
+                The menu category where the menu will be included,
+                if non provided the view will be acessible as a top menu.
             :param category_icon:
                 Font-Awesome icon name for the category, optional.
             :param category_label:
@@ -176,8 +188,10 @@ class BaseApp(object):
                 baseapp = BaseApp(app, db)
                 # Register a view, rendering a top menu without icon.
                 baseapp.add_view(MyGeneralView(), "My View")
+                # or not instantiated
+                baseapp.add_view(MyGeneralView, "My View")
                 # Register a view, a submenu "Other View" from "Other" with a phone icon.
-                baseapp.add_view(MyOtherGeneralView(), "Other View", icon='fa-phone', category="Others")
+                baseapp.add_view(MyOtherGeneralView, "Other View", icon='fa-phone', category="Others")
                 # Register a view, with category icon and translation.
                 baseapp.add_view(YetOtherGeneralView(), "Other View", icon='fa-phone',
                                 label=_('Other View'), category="Others", category_icon='fa-envelop',
@@ -185,7 +199,9 @@ class BaseApp(object):
                 # Add a link
                 baseapp.add_link("google", href="www.google.com", icon = "fa-google-plus")
         """
+        baseview = self._check_and_init(baseview)
         log.info("Registering class %s on menu %s.%s" % (baseview.__class__.__name__, category, name))
+
         if not self._view_exists(baseview):
             baseview.baseapp = self
             self.baseviews.append(baseview)
@@ -195,6 +211,7 @@ class BaseApp(object):
         self.add_link(name=name, href=href, icon=icon, label=label,
                       category=category, category_icon=category_icon,
                       category_label=category_label, baseview=baseview)
+        return baseview
 
     def add_link(self, name, href, icon="", label="", category="", category_icon="", category_label="", baseview=None):
         """
@@ -240,6 +257,9 @@ class BaseApp(object):
                 A BaseView type class instantiated.
                     
         """
+        baseview = self._check_and_init(baseview)
+        log.info("Registering class %s" % (baseview.__class__.__name__))
+
         if not self._view_exists(baseview):
             baseview.baseapp = self
             self.baseviews.append(baseview)
@@ -247,7 +267,21 @@ class BaseApp(object):
             self.register_blueprint(baseview, endpoint=endpoint, static_folder=static_folder)
             self._add_permission(baseview)
         else:
-            log.warning("View alreary exists {0} ignoring".format(baseview.__class__.__name__))
+            log.warning("View already exists {0} ignoring".format(baseview.__class__.__name__))
+        return baseview
+
+    def security_cleanup(self):
+        """
+            This method is useful if you have changed the name of your menus or classes,
+            changing them will leave behind permissions that are not associated with anything.
+
+            You can use it always or just sometimes to
+            perform a security cleanup. Warning this will delete any permission
+            that is no longer part of any registered view or menu.
+
+            Remember invoke ONLY AFTER YOU HAVE REGISTERED ALL VIEWS
+        """
+        self.sm.security_cleanup(self.baseviews, self.menu)
 
     @property
     def get_url_for_login(self):
