@@ -39,8 +39,7 @@ class TemplateFilters(object):
         else:
             args['_oc_' + generalview_name] = column
             args['_od_' + generalview_name] = 'asc'
-        return url_for(request.endpoint,**dict(new_args.items() + args.to_dict().items()))
-
+        return url_for(request.endpoint,**dict(list(new_args.items()) + list(args.to_dict().items())))
 
     @app_template_filter('link_page')
     def link_page_filter(self, page, generalview_name):
@@ -50,7 +49,7 @@ class TemplateFilters(object):
         new_args = request.view_args.copy()
         args = request.args.copy()
         args['page_' + generalview_name] = page
-        return url_for(request.endpoint,**dict(new_args.items() + args.to_dict().items()))
+        return url_for(request.endpoint, **dict(list(new_args.items()) + list(args.to_dict().items())))
 
 
     @app_template_filter('link_page_size')
@@ -61,8 +60,8 @@ class TemplateFilters(object):
         new_args = request.view_args.copy()
         args = request.args.copy()
         args['psize_' + generalview_name] = page_size
-        return url_for(request.endpoint,**dict(new_args.items() + args.to_dict().items()))
-                
+        return url_for(request.endpoint, **dict(list(new_args.items()) + list(args.to_dict().items())))
+
 
     @app_template_filter('get_link_next')
     def get_link_next_filter(self, s):
