@@ -18,7 +18,7 @@ from .models import User, Role, PermissionView, Permission, ViewMenu, \
 from .views import AuthDBView, AuthOIDView, ResetMyPasswordView, AuthLDAPView, \
     ResetPasswordView, UserDBGeneralView, UserLDAPGeneralView, UserOIDGeneralView, RoleGeneralView, \
     PermissionViewGeneralView, ViewMenuGeneralView, PermissionGeneralView, UserStatsChartView
-
+from ..basemanager import BaseManager
 
 log = logging.getLogger(__name__)
 
@@ -33,7 +33,7 @@ ADMIN_USER_FIRST_NAME = 'Admin'
 ADMIN_USER_LAST_NAME = 'User'
 
 
-class SecurityManager(object):
+class SecurityManager(BaseManager):
     session = None
     auth_type = 1
     auth_role_admin = ""
@@ -44,7 +44,7 @@ class SecurityManager(object):
     lm = None
     oid = None
 
-    def __init__(self, app, session):
+    def __init__(self, baseapp):
         """
             SecurityManager contructor
             param app:
@@ -52,7 +52,9 @@ class SecurityManager(object):
             param session:
                 the database session for security tables, passed to BaseApp
         """
-        self.session = session
+        super(SecurityManager, self).__init__(baseapp)
+        self.session = baseapp.get_session
+        app = self.baseapp.get_app
         self.auth_type = self._get_auth_type(app)
         self.auth_role_admin = self._get_auth_role_admin(app)
         self.auth_role_public = self._get_auth_role_public(app)
@@ -68,44 +70,44 @@ class SecurityManager(object):
         self.lm.user_loader(self.load_user)
         self.init_db()
 
-    def register_views(self, baseapp):
-        baseapp.add_view_no_menu(ResetPasswordView())
-        baseapp.add_view_no_menu(ResetMyPasswordView())
+    def register_views(self):
+        self.baseapp.add_view_no_menu(ResetPasswordView())
+        self.baseapp.add_view_no_menu(ResetMyPasswordView())
 
-        if self._get_auth_type(baseapp.app) == AUTH_DB:
-            self.user_view = baseapp._init_view_session(UserDBGeneralView)
+        if self._get_auth_type(self.baseapp.get_app) == AUTH_DB:
+            self.user_view = UserDBGeneralView
             self.auth_view = AuthDBView()
-        elif self._get_auth_type(baseapp.app) == AUTH_LDAP:
-            self.user_view = baseapp._init_view_session(UserLDAPGeneralView)
+        elif self._get_auth_type(self.baseapp.get_app) == AUTH_LDAP:
+            self.user_view = UserLDAPGeneralView
             self.auth_view = AuthLDAPView()
         else:
-            self.user_view = baseapp._init_view_session(UserOIDGeneralView)
+            self.user_view = UserOIDGeneralView
             self.auth_view = AuthOIDView()
             self.oid.after_login_func = self.auth_view.after_login
 
-        baseapp.add_view_no_menu(self.auth_view)
+        self.baseapp.add_view_no_menu(self.auth_view)
 
-        baseapp.add_view(self.user_view, "List Users"
-                         , icon="fa-user", label=_("List Users"),
-                         category="Security", category_icon="fa-cogs", category_label=_('Security'))
+        self.user_view = self.baseapp.add_view(self.user_view, "List Users",
+                                      icon="fa-user", label=_("List Users"),
+                                      category="Security", category_icon="fa-cogs", category_label=_('Security'))
 
-        role_view = baseapp._init_view_session(RoleGeneralView)
-        baseapp.add_view(role_view, "List Roles", icon="fa-group", label=_('List Roles'),
-                         category="Security", category_icon="fa-cogs")
+
+        role_view = self.baseapp.add_view(RoleGeneralView, "List Roles", icon="fa-group", label=_('List Roles'),
+                              category="Security", category_icon="fa-cogs")
         role_view.related_views = [self.user_view.__class__]
-        baseapp.add_view(baseapp._init_view_session(UserStatsChartView),
-                         "User's Statistics", icon="fa-bar-chart-o", label=_("User's Statistics"),
-                         category="Security")
-        baseapp.menu.add_separator("Security")
-        baseapp.add_view(baseapp._init_view_session(PermissionViewGeneralView),
-                         "Base Permissions", icon="fa-lock",
-                         label=_("Base Permissions"), category="Security")
-        baseapp.add_view(baseapp._init_view_session(ViewMenuGeneralView),
-                         "Views/Menus", icon="fa-list-alt",
-                         label=_('Views/Menus'), category="Security")
-        baseapp.add_view(baseapp._init_view_session(PermissionGeneralView), "Permission on Views/Menus",
-                         icon="fa-link", label=_('Permission on Views/Menus'),
-                         category="Security")
+        self.baseapp.add_view(UserStatsChartView,
+                                      "User's Statistics", icon="fa-bar-chart-o", label=_("User's Statistics"),
+                                      category="Security")
+        self.baseapp.menu.add_separator("Security")
+        self.baseapp.add_view(PermissionGeneralView,
+                                      "Base Permissions", icon="fa-lock",
+                                      label=_("Base Permissions"), category="Security")
+        self.baseapp.add_view(ViewMenuGeneralView,
+                                      "Views/Menus", icon="fa-list-alt",
+                                      label=_('Views/Menus'), category="Security")
+        self.baseapp.add_view(PermissionViewGeneralView,
+                                      "Permission on Views/Menus", icon="fa-link",
+                                      label=_('Permission on Views/Menus'), category="Security")
 
 
     def load_user(self, pk):
@@ -115,75 +117,14 @@ class SecurityManager(object):
     def before_request():
         g.user = current_user
 
-    def migrate_get_new_obj(self, old_obj, new_obj):
-        for col in list(old_obj.keys()):
-            setattr(new_obj, col, getattr(old_obj, col))
-        return new_obj
-
-    def migrate_obj(self, old_table, new_class):
-        old_objs = self.session.query(old_table).all()
-        for old_obj in old_objs:
-            new_obj = self.migrate_get_new_obj(old_obj, new_class())
-            self.session.add(new_obj)
-            self.session.commit()
-
-    def quick_mapper(self, table):
-        Base = declarative_base()
-
-        class GenericMapper(Base):
-            __table__ = table
-
-        return GenericMapper
-
-
-    def migrate_db(self):
-        """
-            Migrate security tables from Flask-AppBuilder 0.2.X to 0.3.X
-        """
-        engine = self.session.get_bind(mapper=None, clause=None)
-        inspector = Inspector.from_engine(engine)
-        if 'user' in inspector.get_table_names() and 'role' in inspector.get_table_names() and 'permission' in inspector.get_table_names():
-            log.info("Found previous security tables, migrating...")
-
-            metadata = MetaData(engine)
-
-            old_user = Table('user', metadata, autoload=True)
-            old_role = Table('role', metadata, autoload=True)
-            old_permission = Table('permission', metadata, autoload=True)
-            old_permission_view = Table('permission_view', metadata, autoload=True)
-            old_view_menu = Table('view_menu', metadata, autoload=True)
-            old_permission_view_role = Table('permission_view_role', metadata, autoload=True)
-
-            log.info("Migrating Views and Menus")
-            self.migrate_obj(old_view_menu, ViewMenu)
-
-            log.info("Migrating Permissions")
-            self.migrate_obj(old_permission, Permission)
-
-            log.info("Migrating Permissions on Views")
-            self.migrate_obj(old_permission_view, PermissionView)
-
-            log.info("Migrating Roles")
-            self.migrate_obj(old_role, Role)
-
-            log.info("Migrating Roles to Permissions on Views")
-            self.migrate_obj(old_permission_view_role, self.quick_mapper(assoc_permissionview_role))
-
-            log.info("Migrating Users")
-            self.migrate_obj(old_user, User)
-
-
     def init_db(self):
         try:
             engine = self.session.get_bind(mapper=None, clause=None)
 
-            inspector = Inspector.from_engine(engine)
-            if 'ab_user' not in inspector.get_table_names():
-                log.info("Security DB not found Creating")
-                Base.metadata.create_all(engine)
-                log.info("Security DB Created")
-                self.migrate_db()
-            if self.session.query(Role).filter_by(name=self.auth_role_admin).first() is None:
+            log.info("Security DB not found Creating")
+            Base.metadata.create_all(engine)
+            log.info("Security DB Created")
+            if not self.session.query(Role).filter_by(name=self.auth_role_admin).first():
                 role = Role()
                 role.name = self.auth_role_admin
                 self.session.add(role)
@@ -207,9 +148,11 @@ class SecurityManager(object):
                 self.session.add(user)
                 self.session.commit()
                 log.info("Inserted initial Admin user")
-                log.info("Login using Admin/general")
+                log.info("Login using {0}/{1}".format(ADMIN_USER_NAME, ADMIN_USER_PASSWORD))
         except Exception as e:
-            log.error("DB Creation and initialization failed, if just upgraded to 0.7.X you must migrate the DB. {0}".format(str(e)))
+            log.error(
+                "DB Creation and initialization failed, if just upgraded to 0.7.X you must migrate the DB. {0}".format(
+                    str(e)))
 
 
     """
@@ -217,6 +160,7 @@ class SecurityManager(object):
         AUTHENTICATION METHODS
     ----------------------------------------
     """
+
     def auth_user_db(self, username, password):
         """
             Method for authenticating user, auth db style
@@ -239,6 +183,16 @@ class SecurityManager(object):
             return None
 
     def auth_user_ldap(self, username, password):
+        """
+            Method for authenticating user, auth LDAP style.
+            depends on ldap module that is not mandatory requirement
+            for F.A.B.
+
+            :param username:
+                The username
+            :param password:
+                The password
+        """
         if username is None or username == "":
             return None
         user = self.session.query(User).filter_by(username=username).first()
@@ -397,21 +351,27 @@ class SecurityManager(object):
         return False
 
 
-
     """
         ----------------------------------------
             PERMISSION MANAGEMENT
         ----------------------------------------
     """
 
+    def _find_permission(self, name):
+        """
+            Finds and returns a Permission by name
+        """
+        return self.session.query(Permission).filter_by(name=name).first()
+
+
     def _add_permission(self, name):
         """
             Adds a permission to the backend, model permission
             
             :param name:
-                name of the permission to add: 'can_add','can_edit' etc...
+                name of the permission: 'can_add','can_edit' etc...
         """
-        perm = self.session.query(Permission).filter_by(name=name).first()
+        perm = self._find_permission(name)
         if perm is None:
             try:
                 perm = Permission()
@@ -424,18 +384,42 @@ class SecurityManager(object):
                 self.session.rollback()
         return perm
 
+    def _del_permission(self, name):
+        """
+            Deletes a permission from the backend, model permission
 
-    def _add_view_menu(self, view_name):
+            :param name:
+                name of the permission: 'can_add','can_edit' etc...
+        """
+        perm = self._find_permission(name)
+        if perm:
+            try:
+                self.session.delete(perm)
+                self.session.commit()
+            except Exception as e:
+                log.error("Del Permission Error: {0}".format(str(e)))
+                self.session.rollback()
+
+    #----------------------------------------------
+    #       PERMITIVES VIEW MENU
+    #----------------------------------------------
+    def _find_view_menu(self, name):
+        """
+            Finds and returns a ViewMenu by name
+        """
+        return self.session.query(ViewMenu).filter_by(name=name).first()
+
+    def _add_view_menu(self, name):
         """
             Adds a view or menu to the backend, model view_menu
             param name:
                 name of the view menu to add
         """
-        view_menu = self.session.query(ViewMenu).filter_by(name=view_name).first()
+        view_menu = self._find_view_menu(name)
         if view_menu is None:
             try:
                 view_menu = ViewMenu()
-                view_menu.name = view_name
+                view_menu.name = name
                 self.session.add(view_menu)
                 self.session.commit()
                 return view_menu
@@ -444,6 +428,34 @@ class SecurityManager(object):
                 self.session.rollback()
         return view_menu
 
+    def _del_view_menu(self, name):
+        """
+            Deletes a ViewMenu from the backend
+
+            :param name:
+                name of the ViewMenu
+        """
+        obj = self._find_view_menu(name)
+        if obj:
+            try:
+                self.session.delete(obj)
+                self.session.commit()
+            except Exception as e:
+                log.error("Del Permission Error: {0}".format(str(e)))
+                self.session.rollback()
+
+    #----------------------------------------------
+    #          PERMISSION VIEW MENU
+    #----------------------------------------------
+    def _find_permission_view_menu(self, permission_name, view_menu_name):
+        """
+            Finds and returns a PermissionView by names
+        """
+        permission = self._find_permission(permission_name)
+        view_menu = self._find_view_menu(view_menu_name)
+        return self.session.query(PermissionView).filter_by(permission=permission, view_menu=view_menu).first()
+
+
     def _add_permission_view_menu(self, permission_name, view_menu_name):
         """
             Adds a permission on a view or menu to the backend
@@ -460,7 +472,7 @@ class SecurityManager(object):
         try:
             self.session.add(pv)
             self.session.commit()
-            log.info("Added Permission View %s" % (str(pv)))
+            log.info("Added Permission View: %s" % (str(pv)))
             return pv
         except Exception as e:
             log.error("Add Permission to View Error: {0}".format(str(e)))
@@ -469,24 +481,21 @@ class SecurityManager(object):
 
     def _del_permission_view_menu(self, permission_name, view_menu_name):
         try:
-            perm = self.session.query(Permission).filter_by(name=permission_name).first()
-            vm = self.session.query(ViewMenu).filter_by(name=view_menu_name).first()
-            pv = self.session.query(PermissionView).filter_by(permission=perm, view_menu=vm).first()
+            pv = self._find_permission_view_menu(permission_name, view_menu_name)
             # delete permission on view
             self.session.delete(pv)
             self.session.commit()
-            # if last permission delete permission
-            pv = self.session.query(PermissionView).filter_by(permission=perm).all()
+            # if no more permission on permission view, delete permission
+            pv = self.session.query(PermissionView).filter_by(permission=pv.permission).all()
             if not pv:
-                self.session.delete(perm)
-                self.session.commit()
-            log.info("Removed Permission View %s" % (str(permission_name)))
+                self._del_permission(pv.permission.name)
+            log.info("Removed Permission View: %s" % (str(permission_name)))
         except Exception as e:
             log.error("Del Permission from View Error: {0}".format(str(e)))
             self.session.rollback()
 
 
-    def _find_permission(self, lst, item):
+    def _find_permission_on_views(self, lst, item):
         for i in lst:
             if i.permission.name == item:
                 return True
@@ -498,64 +507,6 @@ class SecurityManager(object):
                 return True
         return False
 
-    def add_permissions_view(self, base_permissions, view_menu):
-        """
-            Adds a permission on a view menu to the backend
-            
-            :param base_permissions:
-                list of permissions from view (all exposed methods): 'can_add','can_edit' etc...
-            :param view_menu:
-                name of the view or menu to add
-        """
-        view_menu_db = self.session.query(ViewMenu).filter_by(name=view_menu).first()
-        if view_menu_db is None:
-            view_menu_db = self._add_view_menu(view_menu)
-        perm_views = self.session.query(PermissionView).filter_by(view_menu_id=view_menu_db.id).all()
-
-        if not perm_views:
-            # No permissions yet on this view
-            for permission in base_permissions:
-                pv = self._add_permission_view_menu(permission, view_menu)
-                role_admin = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
-                self.add_permission_role(role_admin, pv)
-        else:
-            # Permissions on this view exist but....
-            role_admin = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
-            for permission in base_permissions:
-                # Check if base view permissions exist
-                if not self._find_permission(perm_views, permission):
-                    pv = self._add_permission_view_menu(permission, view_menu)
-                    self.add_permission_role(role_admin, pv)
-            for perm_view in perm_views:
-                if perm_view.permission.name not in base_permissions:
-                    # perm to delete
-                    roles = self.session.query(Role).all()
-                    perm = self.session.query(Permission).filter_by(name=perm_view.permission.name).first()
-                    # del permission from all roles
-                    for role in roles:
-                        self.del_permission_role(role, perm)
-                    self._del_permission_view_menu(perm_view.permission.name, view_menu)
-                elif perm_view not in role_admin.permissions:
-                    # Role Admin must have all permissions
-                    self.add_permission_role(role_admin, perm_view)
-
-
-    def add_permissions_menu(self, view_menu_name):
-        """
-            Adds menu_access to menu on permission_view_menu
-            
-            :param view_menu_name:
-                The menu name
-        """
-        view_menu = self.session.query(ViewMenu).filter_by(name=view_menu_name).first()
-        if view_menu is None:
-            view_menu = self._add_view_menu(view_menu_name)
-        lst = self.session.query(PermissionView).filter_by(view_menu_id=view_menu.id).all()
-        if not lst:
-            pv = self._add_permission_view_menu('menu_access', view_menu_name)
-            role_admin = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
-            self.add_permission_role(role_admin, pv)
-
     def add_permission_role(self, role, perm_view):
         """
             Add permission-ViewMenu object to Role
@@ -594,3 +545,80 @@ class SecurityManager(object):
             except Exception as e:
                 log.error("Del Permission to Role Error: {0}".format(str(e)))
                 self.session.rollback()
+
+
+    def add_permissions_view(self, base_permissions, view_menu):
+        """
+            Adds a permission on a view menu to the backend
+
+            :param base_permissions:
+                list of permissions from view (all exposed methods): 'can_add','can_edit' etc...
+            :param view_menu:
+                name of the view or menu to add
+        """
+        view_menu_db = self._add_view_menu(view_menu)
+        perm_views = self.session.query(PermissionView).filter_by(view_menu_id=view_menu_db.id).all()
+
+        if not perm_views:
+            # No permissions yet on this view
+            for permission in base_permissions:
+                pv = self._add_permission_view_menu(permission, view_menu)
+                role_admin = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
+                self.add_permission_role(role_admin, pv)
+        else:
+            # Permissions on this view exist but....
+            role_admin = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
+            for permission in base_permissions:
+                # Check if base view permissions exist
+                if not self._find_permission_on_views(perm_views, permission):
+                    pv = self._add_permission_view_menu(permission, view_menu)
+                    self.add_permission_role(role_admin, pv)
+            for perm_view in perm_views:
+                if perm_view.permission.name not in base_permissions:
+                    # perm to delete
+                    roles = self.session.query(Role).all()
+                    perm = self._find_permission(perm_view.permission.name)
+                    # del permission from all roles
+                    for role in roles:
+                        self.del_permission_role(role, perm)
+                    self._del_permission_view_menu(perm_view.permission.name, view_menu)
+                elif perm_view not in role_admin.permissions:
+                    # Role Admin must have all permissions
+                    self.add_permission_role(role_admin, perm_view)
+
+
+    def add_permissions_menu(self, view_menu_name):
+        """
+            Adds menu_access to menu on permission_view_menu
+
+            :param view_menu_name:
+                The menu name
+        """
+        self._add_view_menu(view_menu_name)
+        pv = self._find_permission_view_menu('menu_access', view_menu_name)
+        if not pv:
+            pv = self._add_permission_view_menu('menu_access', view_menu_name)
+            role_admin = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
+            self.add_permission_role(role_admin, pv)
+
+
+    def security_cleanup(self, baseviews, menus):
+        viewsmenus = self.session.query(ViewMenu).all()
+        roles = self.session.query(Role).all()
+        for viewmenu in viewsmenus:
+            found = False
+            for baseview in baseviews:
+                if viewmenu.name == baseview.__class__.__name__:
+                    found = True
+                    break
+            if menus.find(viewmenu.name):
+                found = True
+            if not found:
+                permissions = self.session.query(PermissionView).filter_by(view_menu_id=viewmenu.id).all()
+                for permission in permissions:
+                    for role in roles:
+                        self.del_permission_role(role, permission)
+                    self._del_permission_view_menu(permission.permission.name, viewmenu.name)
+                self.session.delete(viewmenu)
+                self.session.commit()
+
