@@ -1,24 +1,28 @@
 import datetime
-from ..fieldwidgets import BS3PasswordFieldWidget
+import logging
 from flask import render_template, flash, redirect, session, url_for, request, g
 from werkzeug.security import generate_password_hash
-#from openid.consumer import discover
 from openid.consumer.consumer import Consumer, SUCCESS, CANCEL
 #from openid.extensions import ax
 #from openid.extensions.sreg import SRegRequest, SRegResponse
 from flask.ext.openid import SessionWrapper, OpenIDResponse
-from flask_wtf import EqualTo, PasswordField, validators
+from wtforms import validators, PasswordField
+from wtforms.validators import EqualTo
 from flask.ext.babelpkg import gettext, lazy_gettext
 from flask_login import login_user, logout_user
 
 from flask_appbuilder.models.datamodel import SQLAModel
 from flask_appbuilder.views import BaseView, GeneralView, SimpleFormView, expose
 from flask_appbuilder.charts.views import DirectChartView
+
+from ..fieldwidgets import BS3PasswordFieldWidget
 from ..actions import action
 from .._compat import as_unicode
-from forms import LoginForm_db, LoginForm_oid, ResetPasswordForm
-from models import User, Permission, PermissionView, Role, ViewMenu
-from decorators import has_access
+from .forms import LoginForm_db, LoginForm_oid, ResetPasswordForm
+from .models import User, Permission, PermissionView, Role, ViewMenu
+from .decorators import has_access
+
+log = logging.getLogger(__name__)
 
 
 class PermissionGeneralView(GeneralView):
